from urllib.parse import quote, unquote


class MUrl:
    """url工具类"""

    @classmethod
    def get_url_domain(cls, url: str) -> str:
        """获取url的域名"""

        try:
            return url.split("//")[1].split("/")[0]
        except Exception:
            return ""

    @classmethod
    def get_url_path(cls, url: str) -> str:
        """获取url的路径"""

        path = url.split("//")[1].split("/", 1)  # Split only once
        if len(path) > 1:
            return "/" + path[1]
        return "/"

    @classmethod
    def get_url_params(cls, url: str, use_unquote: bool = True) -> dict:
        """获取url的参数"""

        try:
            params = {}
            for param in url.split("?")[1].split("&"):
                k, v = param.split("=")
                params[k] = v if not use_unquote else unquote(v)
        except Exception:
            return {}
        else:
            return params

    @classmethod
    def drop_url_anchor(cls, url: str) -> str:
        """去除url中的锚点"""

        if url.startswith("#"):
            return ""
        return url.split("#")[0]

    @classmethod
    def is_relative_url(cls, url: str) -> bool:
        """判断url是否为相对路径"""

        return url.startswith(("/", "#"))



from typing import Dict, Optional, Union

from .fetch import async_fetch, fetch

try:
    import ujson as json
except ImportError:
    import json


class Response:
    def __init__(self, text: str, status_code: int):
        self.text = text
        self.status_code = status_code

    def __str__(self):
        return f"Response(text={self.text}, status_code={self.status_code})"

    def __repr__(self):
        return self.__str__()

    def json(self):
        return json.loads(self.text)


class Mxios:
    def __init__(
        self,
        base_url: str = "",
        headers: Optional[Dict] = None,
        proxy_server: str = "",
        timeout: int = 60,
    ):
        self.base_url = base_url if not base_url.endswith("/") else base_url[:-1]
        self.headers = headers or {}
        self.proxy_server = proxy_server
        self.timeout = timeout

    def request(
        self,
        method: str,
        url: str,
        params: Optional[Dict] = None,
        headers: Optional[Dict] = None,
        data: Union[str, Dict] = "{}",
        proxy_server: str = "",
        timeout: int = 60,
    ) -> Response:
        """发起请求"""

        if headers is None:
            headers = {}
        if params is None:
            params = {}
        if isinstance(data, (dict, list)):
            data = json.dumps(data)

        if not (url.startswith(("http://", "https://"))):
            url = self.base_url + url

        headers = {**self.headers, **headers}

        resp_status_code, resp_text = fetch(
            url=url,
            method=method,
            params=params,
            data=data,
            headers=headers,
            proxy_server=self.proxy_server or proxy_server,
            timeout=timeout,
        )

        return Response(resp_text, resp_status_code)

    async def async_request(
        self,
        method: str,
        url: str,
        params: Optional[Dict] = None,
        headers: Optional[Dict] = None,
        data: Union[str, Dict] = "{}",
        proxy_server: str = "",
        timeout: int = 60,
    ) -> Response:
        """发起异步请求"""

        if headers is None:
            headers = {}
        if params is None:
            params = {}
        if isinstance(data, (dict, list)):
            data = json.dumps(data)

        if not (url.startswith(("http://", "https://"))):
            url = self.base_url + url

        headers = {**self.headers, **headers}

        resp_status_code, resp_text = await async_fetch(
            url=url,
            method=method,
            params=params,
            data=data,
            headers=headers,
            proxy_server=self.proxy_server or proxy_server,
            timeout=timeout,
        )

        return Response(resp_text, resp_status_code)

    def get(self, url: str, params: Optional[Dict] = None, headers: Optional[Dict] = None, proxy_server: str = "", timeout: int = 60) -> Response:
        return self.request(method="get", url=url, params=params, headers=headers, proxy_server=proxy_server, timeout=timeout)

    async def async_get(self, url: str, params: Optional[Dict] = None, headers: Optional[Dict] = None, proxy_server: str = "", timeout: int = 60) -> Response:
        return await self.async_request(method="get", url=url, params=params, headers=headers, proxy_server=proxy_server, timeout=timeout)
    
    def post(self, url: str, params: Optional[Dict] = None, headers: Optional[Dict] = None, data: Union[str, Dict] = "{}", proxy_server: str = "", timeout: int = 60) -> Response:
        return self.request(method="post", url=url, params=params, headers=headers, data=data, proxy_server=proxy_server, timeout=timeout)
    
    async def async_post(self, url: str, params: Optional[Dict] = None, headers: Optional[Dict] = None, data: Union[str, Dict] = "{}", proxy_server: str = "", timeout: int = 60) -> Response:
        return await self.async_request(method="post", url=url, params=params, headers=headers, data=data, proxy_server=proxy_server, timeout=timeout)
    
    def put(self, url: str, params: Optional[Dict] = None, headers: Optional[Dict] = None, data: Union[str, Dict] = "{}", proxy_server: str = "", timeout: int = 60) -> Response:
        return self.request(method="put", url=url, params=params, headers=headers, data=data, proxy_server=proxy_server, timeout=timeout)
    
    async def async_put(self, url: str, params: Optional[Dict] = None, headers: Optional[Dict] = None, data: Union[str, Dict] = "{}", proxy_server: str = "", timeout: int = 60) -> Response:
        return await self.async_request(method="put", url=url, params=params, headers=headers, data=data, proxy_server=proxy_server, timeout=timeout)
    
    def delete(self, url: str, params: Optional[Dict] = None, headers: Optional[Dict] = None, data: Union[str, Dict] = "{}", proxy_server: str = "", timeout: int = 60) -> Response:
        return self.request(method="delete", url=url, params=params, headers=headers, data=data, proxy_server=proxy_server, timeout=timeout)
    
    async def async_delete(self, url: str, params: Optional[Dict] = None, headers: Optional[Dict] = None, data: Union[str, Dict] = "{}", proxy_server: str = "", timeout: int = 60) -> Response:
        return await self.async_request(method="delete", url=url, params=params, headers=headers, data=data, proxy_server=proxy_server, timeout=timeout)
    
    def patch(self, url: str, params: Optional[Dict] = None, headers: Optional[Dict] = None, data: Union[str, Dict] = "{}", proxy_server: str = "", timeout: int = 60) -> Response:
        return self.request(method="patch", url=url, params=params, headers=headers, data=data, proxy_server=proxy_server, timeout=timeout)
    
    async def async_patch(self, url: str, params: Optional[Dict] = None, headers: Optional[Dict] = None, data: Union[str, Dict] = "{}", proxy_server: str = "", timeout: int = 60) -> Response:
        return await self.async_request(method="patch", url=url, params=params, headers=headers, data=data, proxy_server=proxy_server, timeout=timeout)




from typing import Dict, Optional, Tuple, Union

import aiohttp
import requests


def fetch(
    url,
    method: str = "get",
    params: Optional[Dict] = None,
    data: str = "",
    headers: Optional[Dict] = None,
    proxy_server: str = "",
    timeout: int = 60,
) -> Tuple[int, str]:
    """发起请求"""

    if headers is None:
        headers = {}
    if params is None:
        params = {}

    if proxy_server:
        proxies = {
            "http": f"http://{proxy_server}",
            "https": f"https://{proxy_server}",
        }
    else:
        proxies = None

    resp = getattr(requests, method)(
        url,
        params=params,
        data=data,
        headers=headers,
        proxies=proxies,
        timeout=timeout,
    )
    return resp.status_code, resp.text


async def async_fetch(
    url,
    method: str = "get",
    params: Optional[Dict] = None,
    data: str = "",
    headers: Optional[Dict] = None,
    proxy_server: str = "",
    timeout: int = 60,
) -> Tuple[int, str]:
    """发起异步请求"""

    if headers is None:
        headers = {}
    if params is None:
        params = {}

    async with aiohttp.ClientSession(headers=headers) as session:
        if proxy_server:
            conn = aiohttp.TCPConnector(limit=10, verify_ssl=False)
            session = aiohttp.ClientSession(connector=conn)
            session._default_headers.update(  # noqa: SLF001
                {"Proxy-Switch-Ip": "yes"},
            )
            session._default_headers.update(  # noqa: SLF001
                {"Proxy-Server": proxy_server},
            )
        async with getattr(session, method)(
            url,
            params=params,
            data=data,
            timeout=timeout,
        ) as resp:
            return resp.status, await resp.text()



from pathlib import Path
from typing import Any, Dict

try:
    import ujson as json
except ImportError:
    import json


class MFile:
    @classmethod
    def ensure_path(cls, path: str):
        """确保路径目录存在

        :param path: 路径
        :param use_parent: 是否使用父目录
        """

        Path(path).mkdir(parents=True, exist_ok=True)
        return path

    @classmethod
    def ensure_dir(cls, root: str):
        """确保路径根目录存在"""

        Path(root).parent.mkdir(parents=True, exist_ok=True)
        return root

    @classmethod
    def read(cls, path: str, encoding: str = "utf-8") -> str:
        """读取文件"""

        with Path.open(Path(path), encoding=encoding) as f:
            return f.read()

    @classmethod
    def read_json(cls, path: str, encoding: str = "utf-8") -> Dict:
        """读取json文件"""

        return json.loads(cls.read(path, encoding=encoding))

    @classmethod
    def write(cls, path: str, content: str, encoding: str = "utf-8"):
        """写入文件"""

        with Path.open(Path(path), "w", encoding=encoding) as f:
            f.write(content)

    @classmethod
    def write_json(cls, path: str, content: Any, encoding: str = "utf-8"):
        """写入json文件"""

        cls.write(path, json.dumps(content), encoding=encoding)

    @classmethod
    def append(cls, path: str, content: str, encoding: str = "utf-8"):
        """追加写入文件"""

        with Path.open(Path(path), "a", encoding=encoding) as f:
            f.write(content)
